import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../flutter_mylibrary.dart';
import 'base_view_state.dart';

enum LoadType {
  Refresh,
  LoadMore,
}

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }

  bool _hasMore;

  set hasMore(bool hasMore) => _hasMore = hasMore;

  bool get hasMore{
   if(_hasMore != null){
     return _hasMore;
   }else{
     return getRequestInfo().useLoadMore? totalNum > list.length? true : false : false;
   }
  }



  List<T> getDataList(HttpData httpData){
    return null;
  }


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  Future<HttpData> getData(LoadType loadType) async{

    HttpData httpData = await request((httpData, requestInfo) {
      if(currentPageNum == firstPage()){
        list.clear();
      }

      List<T> resList = [];

      if(httpData.data is List){
        resList = List<T>.from(httpData.data);
      }else{
        resList = getDataList(httpData);
      }
      list.addAll(resList);
      requestInfo.onSuccess?.call(httpData);
    }, isSetError: currentPageNum == firstPage());

    if(isEmpty(httpData.httpError)){
      setIdle();
    }

    bool success = true;

    if(isNotEmpty(httpData.httpError)){
      if(loadType == LoadType.LoadMore){
        _currentPageNum = --_currentPageNum;
      }
      if(httpData.httpError.httpErrorType != HttpErrorType.EmptyDataError){
        success = false;
      }
    }

    refreshController.finishRefresh(success: success);
    refreshController.finishLoad(success: success, noMore: !hasMore);

    return httpData;
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      getData(LoadType.Refresh);
    }
  }




  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  Future<HttpData> refreshData() {
    _currentPageNum = firstPage();
    return getData(LoadType.Refresh);
  }


  /// 上拉加载更多
  Future<HttpData> loadMore() {
    _currentPageNum = ++_currentPageNum;
    return getData(LoadType.LoadMore);
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  Widget buildEasyRefreshWidget({
    ScrollController scrollController,
    Widget footWidget,
    ViewStateWidgetBuilder emptyWidgetBuilder,
    ViewStateWidgetBuilder errorWidgetBuilder,
    ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder,
    ViewStateModelWidgetBuilder<List<T>> widgetBuilder,
    Widget busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
    }){
      return buildViewStateEasyRefresh<List<T>>(
        data: list,
        scrollController: scrollController,
        footWidget: footWidget,
        errorWidgetBuilder: errorWidgetBuilder,
        emptyWidgetBuilder: emptyWidgetBuilder,
        widgetListBuilder: widgetListBuilder,
        widgetBuilder: widgetBuilder,
        busyWidget: busyWidget,
        showBusyWidget: showBusyWidget,
        showEmptyWidget: showEmptyWidget,
        showErrorWidget: showErrorWidget,
        onLoad: hasMore? onLoad??(){
          loadMore();
        } : null,
        onRefresh: onRefresh??(){
          refreshData();
        }
      );
  }




}
